@@ -36,6 +36,7 @@
 #include "nm-device-wifi.h"
 #include "nm-utils.h"
 #include "nm-active-connection.h"
+#include "nm-vpn-connection.h"
 #include "nm-setting-ip4-config.h"
 
 static gboolean
@@ -202,7 +203,7 @@ dump_wireless (NMDeviceWifi *device)
 
 	g_print ("AccessPoints:\n");
 	aps = nm_device_wifi_get_access_points (device);
-	for (i = 0; i < aps->len; i++) {
+	for (i = 0; aps && (i < aps->len); i++) {
 		dump_access_point (NM_ACCESS_POINT (g_ptr_array_index (aps, i)));
 		g_print ("\n");
 	}
@@ -291,8 +292,10 @@ active_connections_changed (NMClient *client, GParamSpec *pspec, gpointer user_d
 		connection = g_ptr_array_index (connections, i);
 		g_print ("    %s\n", nm_object_get_path (NM_OBJECT (connection)));
 		devices = nm_active_connection_get_devices (connection);
-		for (j = 0; j < devices->len; j++)
+		for (j = 0; devices && j < devices->len; j++)
 			g_print ("           %s\n", nm_device_get_udi (g_ptr_array_index (devices, j)));
+		if (NM_IS_VPN_CONNECTION (connection))
+			g_print ("           VPN base connection: %s\n", nm_active_connection_get_specific_object (connection));
 	}
 }
 
@@ -317,7 +320,8 @@ test_get_active_connections (NMClient *client)
 
 		g_print ("    %s\n", nm_object_get_path (g_ptr_array_index (connections, i)));
 		devices = nm_active_connection_get_devices (g_ptr_array_index (connections, i));
-		g_ptr_array_foreach ((GPtrArray *) devices, show_active_connection_device, NULL);
+		if (devices)
+			g_ptr_array_foreach ((GPtrArray *) devices, show_active_connection_device, NULL);
 	}
 }
 
