@@ -67,11 +67,11 @@ GType nm_vpn_manager_get_type (void);
 
 NMVPNManager *nm_vpn_manager_get (void);
 
-const char *nm_vpn_manager_activate_connection (NMVPNManager *manager,
-                                                NMConnection *connection,
-                                                NMActRequest *act_request,
-                                                NMDevice *device,
-                                                GError **error);
+NMVPNConnection *nm_vpn_manager_activate_connection (NMVPNManager *manager,
+                                                     NMConnection *connection,
+                                                     NMActRequest *act_request,
+                                                     NMDevice *device,
+                                                     GError **error);
 
 gboolean nm_vpn_manager_deactivate_connection (NMVPNManager *manager,
                                                const char *path,
@@ -83,4 +83,7 @@ void nm_vpn_manager_add_active_connections (NMVPNManager *manager,
 
 GSList *nm_vpn_manager_get_active_connections (NMVPNManager *manager);
 
+NMConnection *nm_vpn_manager_get_connection_for_active (NMVPNManager *manager,
+                                                        const char *active_path);
+
 #endif /* NM_VPN_VPN_MANAGER_H */
